@@ -28,6 +28,7 @@
 #include "low_interface.h"
 #include "utils.h"
 #include "cdio/mmc.h"
+#include "cdio/util.h"
 #include <limits.h>
 #include <ctype.h>
 
@@ -109,6 +110,7 @@ cdio_cddap_find_a_cdrom(int messagedest, char **ppsz_messages){
 	if((d=cdda_identify(buffer, messagedest, ppsz_messages)))
 	  return(d);
 	idmessage(messagedest, ppsz_messages, "", NULL);
+	free(buffer);
       }
     }else{
       /* Name.  Go for it. */
@@ -145,7 +147,7 @@ test_resolve_symlink(const char *file, int messagedest, char **ppsz_messages)
     return(NULL);
   }
 
-  if (realpath(file,resolved))
+  if (cdio_realpath(file,resolved))
     return(strdup(resolved));
 
   idperror(messagedest, ppsz_messages, "\t\tCould not resolve symlink %s",
@@ -323,7 +325,6 @@ cdda_identify_device_cdio(CdIo_t *p_cdio, const char *psz_device,
 	snprintf( d->drive_model, i_len, "%s %s %s %s", 
 		  hw_info.psz_vendor, hw_info.psz_model, hw_info.psz_revision,
 		  description );
-	free(description);
       } else {
 	d->drive_model=malloc( i_len );
 	snprintf( d->drive_model, i_len, "%s %s %s", 
@@ -335,5 +336,8 @@ cdda_identify_device_cdio(CdIo_t *p_cdio, const char *psz_device,
     }
   }
   
+  if (description)
+    free(description);
+
   return(d);
 }
