@@ -1,26 +1,23 @@
 /*
-    $Id: aix.c,v 1.2 2006/03/17 19:06:51 rocky Exp $
+  Copyright (C) 2004, 2005, 2006, 2008, 2010, 2011 Rocky Bernstein <rocky@gnu.org>
 
-    Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301 USA.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #ifdef HAVE_STRING_H
@@ -38,7 +35,7 @@
 
 #ifdef HAVE_AIX_CDROM
 
-static const char _rcsid[] = "$Id: aix.c,v 1.2 2006/03/17 19:06:51 rocky Exp $";
+static const char _rcsid[] = "$Id: aix.c,v 1.3 2008/04/22 15:29:11 karl Exp $";
 
 #ifdef HAVE_GLOB_H
 #include <glob.h>
@@ -681,6 +678,13 @@ _cdio_malloc_and_zero(size_t size) {
 }
 #endif
 
+static bool
+is_mmc_supported(void *user_data)
+{
+    _img_private_t *env = user_data;
+    return (_AM_NONE == env->access_mode) ? false : true;
+}
+
 /*!
   Return the value associated with the key "arg".
 */
@@ -698,6 +702,8 @@ get_arg_aix (void *p_user_data, const char key[])
     case _AM_NONE:
       return "no access method";
     }
+  } else if (!strcmp (key, "mmc-supported?")) {
+      return is_mmc_supported(env) ? "true" : "false";
   } 
   return NULL;
 }
