@@ -1,7 +1,5 @@
 /*
-  $Id: iso9660_fs.c,v 1.47 2008/04/18 16:02:09 karl Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2011
     Rocky Bernstein <rocky@gnu.org>
   Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -20,8 +18,9 @@
 */
 /* iso9660 filesystem-based routines */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H) && !defined(__CDIO_CONFIG_H__)
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #ifdef HAVE_STRING_H
@@ -192,10 +191,9 @@ iso9660_open_ext_private (const char *psz_path,
   return p_iso;
 
  error:
-  if (p_iso && p_iso->stream) {
-    cdio_stdio_destroy(p_iso->stream);
-    free(p_iso);
-  }
+  if (p_iso && p_iso->stream) cdio_stdio_destroy(p_iso->stream);
+  free(p_iso);
+  
   return NULL;
 }
 
@@ -777,7 +775,7 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
 	  calloc(1, sizeof(iso9660_stat_t)+i_rr_fname+2);
         if (!p_stat_new)
           {
-          cdio_warn("Couldn't calloc(1, %d)", sizeof(iso9660_stat_t)+i_rr_fname+2);
+          cdio_warn("Couldn't calloc(1, %zd)", sizeof(iso9660_stat_t)+i_rr_fname+2);
           return NULL;
           }
 	memcpy(p_stat_new, p_stat, stat_len);
@@ -800,6 +798,7 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
           free(p_psz_out);
         }
         else {
+          free(p_stat);
           return NULL;
         }
       }
@@ -1029,6 +1028,7 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes", 
 		      (long unsigned int) strlen(p_stat->filename));
+	    free(p_stat);
 	    return NULL;
 	  }
 	  trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname,
@@ -1135,6 +1135,7 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes", 
 		      (long unsigned int) strlen(p_stat->filename));
+	    free(p_stat);
 	    return NULL;
 	  }
 	  trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname, 
