@@ -1,7 +1,6 @@
 /*
-  $Id: util.c,v 1.54 2008/04/14 17:30:27 karl Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2010
+  Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,6 +26,13 @@
 # include <sys/stat.h>
 #endif
 
+#ifndef HAVE_S_ISLNK
+# define S_ISLNK(s) ((void)s,0)
+#endif
+#ifndef HAVE_S_ISSOCK
+# define S_ISSOCK(s) ((void)s,0)
+#endif
+
 cdio_log_handler_t gl_default_cdio_log_handler = NULL;
 char *source_name = NULL;
 char *program_name;
@@ -45,15 +51,16 @@ print_version (char *program_name, const char *version,
 	       int no_header, bool version_only)
 {
   
-  if (no_header == 0)
+  if (no_header == 0) {
     report( stdout,  
-	    "%s version %s\nCopyright (c) 2003, 2004, 2005, 2007, 2008 R. Bernstein\n",
+	    "%s version %s\nCopyright (c) 2003, 2004, 2005, 2007, 2008, 2011 R. Bernstein\n",
 	    program_name, version);
-  report( stdout,  
-	  _("This is free software; see the source for copying conditions.\n\
+    report( stdout,  
+            _("This is free software; see the source for copying conditions.\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 PARTICULAR PURPOSE.\n\
 "));
+  }
 
   if (version_only) {
     char *default_device;
